
myptf<- function(ptfcode,ptfname=ptfcode,prices=NULL,quantities=NULL,currency='EUR') {
  if (!is.timeSeries(prices)) {
  	stop('Object of class "timeSeries" required for argument "prices"')
  }
  
  if (!is.null(quantities)) {
    if (!is.timeSeries(quantities)) { quantities<- as.timeSeries(quantities) }
    if (ncol(quantities)!=ncol(prices)) {
      stop('Numbers of columns of "prices" and "quantities" do not correspond')
    }
  }
  
  out<- list(code=as.character(ptfcode),
             name=as.character(ptfname),
             prices=prices,
             quantities=quantities,
             currency=as.character(currency))
  class(out)<- 'myptf'
  out
}




print.myptf<- function(x,...) {
  cat(paste0(rep('-',50),collapse=''),'\n')
  cat(' Object of class "myptf"\n')
  cat(paste0(rep('-',50),collapse=''),'\n')
  cat(' Code             :',x$code,'\n')
  cat(' Name             :',x$name,'\n')
  cat(' Number of assets :',ncol(x$prices),'\n')
  cat(' Currency         :',x$currency,'\n')
  cat(' Current value    :',round(get_value(x),2),x$currency,'\n')
  cat(paste0(rep('-',50),collapse=''),'\n')
  cat('\n')
}





get_zero_obs<- function(x,date) {
  timeSeries(matrix(0,1,ncol(x)),as.Date(date),colnames(x))
}




update_quantities<- function(x,newquantities) {
  if (!inherits(x,'myptf')) {
    stop('Object of class "myptf" required for argument "x"')
  }
  
  depo<- colnames(x$quantities)
  x$quantities<- rbind(x$quantities,as.timeSeries(newquantities))
  colnames(x$quantities)<- depo
  x$quantities<- sort(x$quantities)
  x
}




update_prices<- function(x,newprices) {
  if (!inherits(x,'myptf')) {
    stop('Object of class "myptf" required for argument "x"')
  }
  
  depo<- colnames(x$prices)
  x$quantities<- rbind(x$prices,as.timeSeries(newprices))
  colnames(x$prices)<- depo
  x$prices<- sort(x$prices)
  x
}


get_quantities<- function(x,date=Sys.Date()) {
  if (!inherits(x,'myptf')) {
    stop('Object of class "myptf" required for argument "x"')
  }
  
  out<- window(x$quantities,start(x$quantities),as.Date(date))

  if (nrow(out)==0) { out<- get_zero_obs(out,date) }

  out<- sort(out,decreasing=TRUE)
  
  out[1,]
}



get_prices<- function(x,date=Sys.Date()) {
  if (!inherits(x,'myptf')) {
    stop('Object of class "myptf" required for argument "x"')
  }
  
  out<- window(x$prices,start(x$prices),as.Date(date))

  if (nrow(out)==0) { out<- get_zero_obs(out,date) }

  out<- sort(out,decreasing=TRUE)
  
  out[1,]
}


get_value<- function(x,date=Sys.Date()) {
  sum(as.numeric(get_quantities(x,date))*as.numeric(get_prices(x,date)))
}



get_hist_value<- function(x) {
  date<- as.Date(rownames(window(x$prices,start(x$quantities),Sys.Date())))
  values<- sapply(date,get_value,x=x)
  timeSeries(values,date,x$code)
}



get_composition<- function(x,date=Sys.Date(),value=FALSE) {
  if (!inherits(x,'myptf')) {
    stop('Object of class "myptf" required for argument "x"')
  }
  out<- as.numeric(get_quantities(x,date))*as.numeric(get_prices(x,date))
  if (!value) { out<- out/sum(out) }
  names(out)<- colnames(x$prices)
  out
}



